<?php
namespace SOSIDEE_DYNAMIC_QRCODE\SRC\FORM;
defined( 'SOSIDEE_DYNAMIC_QRCODE' ) or die( 'you were not supposed to be here' );

use \SOSIDEE_DYNAMIC_QRCODE\SOS\WP\DATA as DATA;
use \SOSIDEE_DYNAMIC_QRCODE\SRC as SRC;

class Config
{
    private $native;

    public $urlError;
    public $urlInactive;
    public $urlExpired;
    public $urlFinished;

    public $mfaEnabled;
    public $anyDeviceEnabled;
    public $logDisabled;
    public $sharedCodeEnabled;
    public $anyQrHideEnabled;
    public $imgSize;
    public $imgPad;
    public $formCheckMode;


    public function __construct($section) {

        $this->native = $section;

        $this->urlError = $this->addUrlField('url-error', 'Redirect URL for disabled/invalid keys');
        $this->urlInactive = $this->addUrlField('url-inactive', 'Redirect URL for inactive keys');
        $this->urlExpired = $this->addUrlField('url-expired', 'Redirect URL for expired keys');
        $this->urlFinished = $this->addUrlField('url-finished', 'Redirect URL for finished keys');

        $this->anyDeviceEnabled = $this->native->addField( 'anydevice-enabled', 'Enable requests from any type of device', false, DATA\FieldType::CHECK );

        $this->imgSize = $this->native->addField( 'image-size', 'QR image size', 256, DATA\FieldType::NUMBER );

        $this->imgPad = $this->native->addField( 'image-pad', 'QR image border', 0, DATA\FieldType::NUMBER );

        $this->sharedCodeEnabled = $this->native->addField( 'shared-code-enabled', 'Disable unique keys', false, DATA\FieldType::CHECK );

        $this->anyQrHideEnabled = $this->native->addField( 'anyqrhide-enabled', 'Enable content hiding with standard QR code images', false, DATA\FieldType::CHECK );

        $this->formCheckMode = $this->native->addField( 'form-check-mode', 'Form check mode', CheckMode::REFERER, DATA\FieldType::SELECT );

        $this->logDisabled = $this->native->addField( 'log-disabled', 'Disable logs in database', false, DATA\FieldType::CHECK );

        $this->mfaEnabled = $this->native->addField( 'mfa-enabled', 'Enable My FastAPP options', false, DATA\FieldType::CHECK );

    }

    private function addUrlField($key, $title, $value = '') {
        return $this->native->addField($key, $title, $value, DATA\FieldType::COMBOBOX);
    }

    private function initializeUrlField( $field ) {
        $field->options = Base::getOptions();
    }
    private function initialize() {
        $this->initializeUrlField( $this->urlError );
        $this->urlError->description = Base::getDescription("mandatory field");
        $this->initializeUrlField( $this->urlInactive );
        $this->initializeUrlField( $this->urlExpired );
        $this->initializeUrlField( $this->urlFinished );

        $this->anyDeviceEnabled->description = Base::getDescription("if unchecked, only mobile devices are accepted");

        $imgMax = SRC\QrCode::getImageMaxSize();
        $this->imgSize->min = 1;
        $this->imgSize->max = $imgMax;
        $this->imgSize->description = Base::getDescription("max. {$imgMax}");

        $padMax = SRC\QrCode::IMAGE_PAD_MAX;
        $this->imgPad->min = 0;
        $this->imgPad->max = $padMax;
        $this->imgPad->description = Base::getDescription("max. {$padMax} (does not affect the image size)");

        $this->sharedCodeEnabled->description = Base::getDescription("if unchecked: QR-Codes must have unique keys<br>if checked: the same key can be shared among various QR-Codes");

        $this->anyQrHideEnabled->description = Base::getDescription("if unchecked: content hiding is enabled only with the enhanced versions of the QR code images");

        $plugin = \SOSIDEE_DYNAMIC_QRCODE\SosPlugin::instance();
        $this->mfaEnabled->label = ' ' . $plugin->help('mfa', 'vertical-align: text-bottom;');

        $this->formCheckMode->options = CheckMode::getList();
        $this->formCheckMode->description = Base::getDescription('used in post/page with hidden content containing a form');

    }

    public function html() {
        $this->initialize();
        $this->native->html();
    }

    public function setPage($page) {
        $this->native->setPage($page);
    }

    public function getField($key) {
        return $this->native->getField($key);
    }

    public function check() {
        $this->load();
        return $this->urlError->value != ''; // && $this->urlInactive->value != '' && $this->urlExpired->value != '';
    }

    public function load() {
        $this->native->load();
    }


}